from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.assay_run_created_event import AssayRunCreatedEvent
from ..models.assay_run_updated_fields_event import AssayRunUpdatedFieldsEvent
from ..models.automation_input_generator_completed_v2_beta_event import (
    AutomationInputGeneratorCompletedV2BetaEvent,
)
from ..models.automation_output_processor_completed_v2_beta_event import (
    AutomationOutputProcessorCompletedV2BetaEvent,
)
from ..models.automation_output_processor_uploaded_v2_beta_event import (
    AutomationOutputProcessorUploadedV2BetaEvent,
)
from ..models.entity_registered_event import EntityRegisteredEvent
from ..models.entry_created_event import EntryCreatedEvent
from ..models.entry_updated_fields_event import EntryUpdatedFieldsEvent
from ..models.entry_updated_review_record_event import EntryUpdatedReviewRecordEvent
from ..models.request_created_event import RequestCreatedEvent
from ..models.request_updated_fields_event import RequestUpdatedFieldsEvent
from ..models.stage_entry_created_event import StageEntryCreatedEvent
from ..models.stage_entry_updated_fields_event import StageEntryUpdatedFieldsEvent
from ..models.stage_entry_updated_review_record_event import StageEntryUpdatedReviewRecordEvent
from ..models.workflow_output_created_event import WorkflowOutputCreatedEvent
from ..models.workflow_output_updated_fields_event import WorkflowOutputUpdatedFieldsEvent
from ..models.workflow_task_created_event import WorkflowTaskCreatedEvent
from ..models.workflow_task_group_created_event import WorkflowTaskGroupCreatedEvent
from ..models.workflow_task_group_updated_watchers_event import WorkflowTaskGroupUpdatedWatchersEvent
from ..models.workflow_task_updated_assignee_event import WorkflowTaskUpdatedAssigneeEvent
from ..models.workflow_task_updated_fields_event import WorkflowTaskUpdatedFieldsEvent
from ..models.workflow_task_updated_scheduled_on_event import WorkflowTaskUpdatedScheduledOnEvent
from ..models.workflow_task_updated_status_event import WorkflowTaskUpdatedStatusEvent
from ..types import UNSET, Unset

T = TypeVar("T", bound="EventsPaginatedList")


@attr.s(auto_attribs=True, repr=False)
class EventsPaginatedList:
    """  """

    _events: Union[
        Unset,
        List[
            Union[
                EntityRegisteredEvent,
                EntryCreatedEvent,
                EntryUpdatedFieldsEvent,
                EntryUpdatedReviewRecordEvent,
                StageEntryCreatedEvent,
                StageEntryUpdatedFieldsEvent,
                StageEntryUpdatedReviewRecordEvent,
                RequestCreatedEvent,
                RequestUpdatedFieldsEvent,
                AssayRunCreatedEvent,
                AssayRunUpdatedFieldsEvent,
                AutomationInputGeneratorCompletedV2BetaEvent,
                AutomationOutputProcessorCompletedV2BetaEvent,
                AutomationOutputProcessorUploadedV2BetaEvent,
                WorkflowTaskGroupCreatedEvent,
                WorkflowTaskCreatedEvent,
                WorkflowTaskUpdatedFieldsEvent,
                WorkflowTaskUpdatedStatusEvent,
                WorkflowTaskUpdatedAssigneeEvent,
                WorkflowTaskUpdatedScheduledOnEvent,
                WorkflowTaskGroupUpdatedWatchersEvent,
                WorkflowOutputCreatedEvent,
                WorkflowOutputUpdatedFieldsEvent,
                UnknownType,
            ]
        ],
    ] = UNSET
    _next_token: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("events={}".format(repr(self._events)))
        fields.append("next_token={}".format(repr(self._next_token)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "EventsPaginatedList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        events: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._events, Unset):
            events = []
            for events_item_data in self._events:
                if isinstance(events_item_data, UnknownType):
                    events_item = events_item_data.value
                elif isinstance(events_item_data, EntityRegisteredEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, EntryCreatedEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, EntryUpdatedFieldsEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, EntryUpdatedReviewRecordEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, StageEntryCreatedEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, StageEntryUpdatedFieldsEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, StageEntryUpdatedReviewRecordEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, RequestCreatedEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, RequestUpdatedFieldsEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, AssayRunCreatedEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, AssayRunUpdatedFieldsEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, AutomationInputGeneratorCompletedV2BetaEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, AutomationOutputProcessorCompletedV2BetaEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, AutomationOutputProcessorUploadedV2BetaEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, WorkflowTaskGroupCreatedEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, WorkflowTaskCreatedEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, WorkflowTaskUpdatedFieldsEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, WorkflowTaskUpdatedStatusEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, WorkflowTaskUpdatedAssigneeEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, WorkflowTaskUpdatedScheduledOnEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, WorkflowTaskGroupUpdatedWatchersEvent):
                    events_item = events_item_data.to_dict()

                elif isinstance(events_item_data, WorkflowOutputCreatedEvent):
                    events_item = events_item_data.to_dict()

                else:
                    events_item = events_item_data.to_dict()

                events.append(events_item)

        next_token = self._next_token

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if events is not UNSET:
            field_dict["events"] = events
        if next_token is not UNSET:
            field_dict["nextToken"] = next_token

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_events() -> Union[
            Unset,
            List[
                Union[
                    EntityRegisteredEvent,
                    EntryCreatedEvent,
                    EntryUpdatedFieldsEvent,
                    EntryUpdatedReviewRecordEvent,
                    StageEntryCreatedEvent,
                    StageEntryUpdatedFieldsEvent,
                    StageEntryUpdatedReviewRecordEvent,
                    RequestCreatedEvent,
                    RequestUpdatedFieldsEvent,
                    AssayRunCreatedEvent,
                    AssayRunUpdatedFieldsEvent,
                    AutomationInputGeneratorCompletedV2BetaEvent,
                    AutomationOutputProcessorCompletedV2BetaEvent,
                    AutomationOutputProcessorUploadedV2BetaEvent,
                    WorkflowTaskGroupCreatedEvent,
                    WorkflowTaskCreatedEvent,
                    WorkflowTaskUpdatedFieldsEvent,
                    WorkflowTaskUpdatedStatusEvent,
                    WorkflowTaskUpdatedAssigneeEvent,
                    WorkflowTaskUpdatedScheduledOnEvent,
                    WorkflowTaskGroupUpdatedWatchersEvent,
                    WorkflowOutputCreatedEvent,
                    WorkflowOutputUpdatedFieldsEvent,
                    UnknownType,
                ]
            ],
        ]:
            events = []
            _events = d.pop("events")
            for events_item_data in _events or []:

                def _parse_events_item(
                    data: Union[Dict[str, Any]]
                ) -> Union[
                    EntityRegisteredEvent,
                    EntryCreatedEvent,
                    EntryUpdatedFieldsEvent,
                    EntryUpdatedReviewRecordEvent,
                    StageEntryCreatedEvent,
                    StageEntryUpdatedFieldsEvent,
                    StageEntryUpdatedReviewRecordEvent,
                    RequestCreatedEvent,
                    RequestUpdatedFieldsEvent,
                    AssayRunCreatedEvent,
                    AssayRunUpdatedFieldsEvent,
                    AutomationInputGeneratorCompletedV2BetaEvent,
                    AutomationOutputProcessorCompletedV2BetaEvent,
                    AutomationOutputProcessorUploadedV2BetaEvent,
                    WorkflowTaskGroupCreatedEvent,
                    WorkflowTaskCreatedEvent,
                    WorkflowTaskUpdatedFieldsEvent,
                    WorkflowTaskUpdatedStatusEvent,
                    WorkflowTaskUpdatedAssigneeEvent,
                    WorkflowTaskUpdatedScheduledOnEvent,
                    WorkflowTaskGroupUpdatedWatchersEvent,
                    WorkflowOutputCreatedEvent,
                    WorkflowOutputUpdatedFieldsEvent,
                    UnknownType,
                ]:
                    events_item: Union[
                        EntityRegisteredEvent,
                        EntryCreatedEvent,
                        EntryUpdatedFieldsEvent,
                        EntryUpdatedReviewRecordEvent,
                        StageEntryCreatedEvent,
                        StageEntryUpdatedFieldsEvent,
                        StageEntryUpdatedReviewRecordEvent,
                        RequestCreatedEvent,
                        RequestUpdatedFieldsEvent,
                        AssayRunCreatedEvent,
                        AssayRunUpdatedFieldsEvent,
                        AutomationInputGeneratorCompletedV2BetaEvent,
                        AutomationOutputProcessorCompletedV2BetaEvent,
                        AutomationOutputProcessorUploadedV2BetaEvent,
                        WorkflowTaskGroupCreatedEvent,
                        WorkflowTaskCreatedEvent,
                        WorkflowTaskUpdatedFieldsEvent,
                        WorkflowTaskUpdatedStatusEvent,
                        WorkflowTaskUpdatedAssigneeEvent,
                        WorkflowTaskUpdatedScheduledOnEvent,
                        WorkflowTaskGroupUpdatedWatchersEvent,
                        WorkflowOutputCreatedEvent,
                        WorkflowOutputUpdatedFieldsEvent,
                        UnknownType,
                    ]
                    discriminator_value: str = cast(str, data.get("eventType"))
                    if discriminator_value is not None:
                        event: Union[
                            EntityRegisteredEvent,
                            EntryCreatedEvent,
                            EntryUpdatedFieldsEvent,
                            EntryUpdatedReviewRecordEvent,
                            StageEntryCreatedEvent,
                            StageEntryUpdatedFieldsEvent,
                            StageEntryUpdatedReviewRecordEvent,
                            RequestCreatedEvent,
                            RequestUpdatedFieldsEvent,
                            AssayRunCreatedEvent,
                            AssayRunUpdatedFieldsEvent,
                            AutomationInputGeneratorCompletedV2BetaEvent,
                            AutomationOutputProcessorCompletedV2BetaEvent,
                            AutomationOutputProcessorUploadedV2BetaEvent,
                            WorkflowTaskGroupCreatedEvent,
                            WorkflowTaskCreatedEvent,
                            WorkflowTaskUpdatedFieldsEvent,
                            WorkflowTaskUpdatedStatusEvent,
                            WorkflowTaskUpdatedAssigneeEvent,
                            WorkflowTaskUpdatedScheduledOnEvent,
                            WorkflowTaskGroupUpdatedWatchersEvent,
                            WorkflowOutputCreatedEvent,
                            WorkflowOutputUpdatedFieldsEvent,
                            UnknownType,
                        ]
                        if discriminator_value == "v2-alpha.stageEntry.created":
                            event = StageEntryCreatedEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2-beta.automationInputGenerator.completed":
                            event = AutomationInputGeneratorCompletedV2BetaEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2-beta.automationOutputProcessor.completed":
                            event = AutomationOutputProcessorCompletedV2BetaEvent.from_dict(
                                data, strict=False
                            )

                            return event
                        if discriminator_value == "v2-beta.automationOutputProcessor.uploaded":
                            event = AutomationOutputProcessorUploadedV2BetaEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2.assayRun.created":
                            event = AssayRunCreatedEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2.assayRun.updated.fields":
                            event = AssayRunUpdatedFieldsEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2.entity.registered":
                            event = EntityRegisteredEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2.entry.created":
                            event = EntryCreatedEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2.entry.updated.fields":
                            event = EntryUpdatedFieldsEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2.entry.updated.reviewRecord":
                            event = EntryUpdatedReviewRecordEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2.request.created":
                            event = RequestCreatedEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2.request.updated.fields":
                            event = RequestUpdatedFieldsEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2.workflowOutput.created":
                            event = WorkflowOutputCreatedEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2.workflowOutput.updated.fields":
                            event = WorkflowOutputUpdatedFieldsEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2.workflowTask.created":
                            event = WorkflowTaskCreatedEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2.workflowTask.updated.assignee":
                            event = WorkflowTaskUpdatedAssigneeEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2.workflowTask.updated.fields":
                            event = WorkflowTaskUpdatedFieldsEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2.workflowTask.updated.scheduledOn":
                            event = WorkflowTaskUpdatedScheduledOnEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2.workflowTask.updated.status":
                            event = WorkflowTaskUpdatedStatusEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2.workflowTaskGroup.created":
                            event = WorkflowTaskGroupCreatedEvent.from_dict(data, strict=False)

                            return event
                        if discriminator_value == "v2.workflowTaskGroup.updated.watchers":
                            event = WorkflowTaskGroupUpdatedWatchersEvent.from_dict(data, strict=False)

                            return event

                        return UnknownType(value=data)
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = EntityRegisteredEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = EntryCreatedEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = EntryUpdatedFieldsEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = EntryUpdatedReviewRecordEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = StageEntryCreatedEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = StageEntryUpdatedFieldsEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = StageEntryUpdatedReviewRecordEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = RequestCreatedEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = RequestUpdatedFieldsEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = AssayRunCreatedEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = AssayRunUpdatedFieldsEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = AutomationInputGeneratorCompletedV2BetaEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = AutomationOutputProcessorCompletedV2BetaEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = AutomationOutputProcessorUploadedV2BetaEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = WorkflowTaskGroupCreatedEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = WorkflowTaskCreatedEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = WorkflowTaskUpdatedFieldsEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = WorkflowTaskUpdatedStatusEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = WorkflowTaskUpdatedAssigneeEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = WorkflowTaskUpdatedScheduledOnEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = WorkflowTaskGroupUpdatedWatchersEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = WorkflowOutputCreatedEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        event = WorkflowOutputUpdatedFieldsEvent.from_dict(data, strict=True)

                        return event
                    except:  # noqa: E722
                        pass
                    return UnknownType(data)

                events_item = _parse_events_item(events_item_data)

                events.append(events_item)

            return events

        try:
            events = get_events()
        except KeyError:
            if strict:
                raise
            events = cast(
                Union[
                    Unset,
                    List[
                        Union[
                            EntityRegisteredEvent,
                            EntryCreatedEvent,
                            EntryUpdatedFieldsEvent,
                            EntryUpdatedReviewRecordEvent,
                            StageEntryCreatedEvent,
                            StageEntryUpdatedFieldsEvent,
                            StageEntryUpdatedReviewRecordEvent,
                            RequestCreatedEvent,
                            RequestUpdatedFieldsEvent,
                            AssayRunCreatedEvent,
                            AssayRunUpdatedFieldsEvent,
                            AutomationInputGeneratorCompletedV2BetaEvent,
                            AutomationOutputProcessorCompletedV2BetaEvent,
                            AutomationOutputProcessorUploadedV2BetaEvent,
                            WorkflowTaskGroupCreatedEvent,
                            WorkflowTaskCreatedEvent,
                            WorkflowTaskUpdatedFieldsEvent,
                            WorkflowTaskUpdatedStatusEvent,
                            WorkflowTaskUpdatedAssigneeEvent,
                            WorkflowTaskUpdatedScheduledOnEvent,
                            WorkflowTaskGroupUpdatedWatchersEvent,
                            WorkflowOutputCreatedEvent,
                            WorkflowOutputUpdatedFieldsEvent,
                            UnknownType,
                        ]
                    ],
                ],
                UNSET,
            )

        def get_next_token() -> Union[Unset, str]:
            next_token = d.pop("nextToken")
            return next_token

        try:
            next_token = get_next_token()
        except KeyError:
            if strict:
                raise
            next_token = cast(Union[Unset, str], UNSET)

        events_paginated_list = cls(
            events=events,
            next_token=next_token,
        )

        events_paginated_list.additional_properties = d
        return events_paginated_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def events(
        self,
    ) -> List[
        Union[
            EntityRegisteredEvent,
            EntryCreatedEvent,
            EntryUpdatedFieldsEvent,
            EntryUpdatedReviewRecordEvent,
            StageEntryCreatedEvent,
            StageEntryUpdatedFieldsEvent,
            StageEntryUpdatedReviewRecordEvent,
            RequestCreatedEvent,
            RequestUpdatedFieldsEvent,
            AssayRunCreatedEvent,
            AssayRunUpdatedFieldsEvent,
            AutomationInputGeneratorCompletedV2BetaEvent,
            AutomationOutputProcessorCompletedV2BetaEvent,
            AutomationOutputProcessorUploadedV2BetaEvent,
            WorkflowTaskGroupCreatedEvent,
            WorkflowTaskCreatedEvent,
            WorkflowTaskUpdatedFieldsEvent,
            WorkflowTaskUpdatedStatusEvent,
            WorkflowTaskUpdatedAssigneeEvent,
            WorkflowTaskUpdatedScheduledOnEvent,
            WorkflowTaskGroupUpdatedWatchersEvent,
            WorkflowOutputCreatedEvent,
            WorkflowOutputUpdatedFieldsEvent,
            UnknownType,
        ]
    ]:
        if isinstance(self._events, Unset):
            raise NotPresentError(self, "events")
        return self._events

    @events.setter
    def events(
        self,
        value: List[
            Union[
                EntityRegisteredEvent,
                EntryCreatedEvent,
                EntryUpdatedFieldsEvent,
                EntryUpdatedReviewRecordEvent,
                StageEntryCreatedEvent,
                StageEntryUpdatedFieldsEvent,
                StageEntryUpdatedReviewRecordEvent,
                RequestCreatedEvent,
                RequestUpdatedFieldsEvent,
                AssayRunCreatedEvent,
                AssayRunUpdatedFieldsEvent,
                AutomationInputGeneratorCompletedV2BetaEvent,
                AutomationOutputProcessorCompletedV2BetaEvent,
                AutomationOutputProcessorUploadedV2BetaEvent,
                WorkflowTaskGroupCreatedEvent,
                WorkflowTaskCreatedEvent,
                WorkflowTaskUpdatedFieldsEvent,
                WorkflowTaskUpdatedStatusEvent,
                WorkflowTaskUpdatedAssigneeEvent,
                WorkflowTaskUpdatedScheduledOnEvent,
                WorkflowTaskGroupUpdatedWatchersEvent,
                WorkflowOutputCreatedEvent,
                WorkflowOutputUpdatedFieldsEvent,
                UnknownType,
            ]
        ],
    ) -> None:
        self._events = value

    @events.deleter
    def events(self) -> None:
        self._events = UNSET

    @property
    def next_token(self) -> str:
        if isinstance(self._next_token, Unset):
            raise NotPresentError(self, "next_token")
        return self._next_token

    @next_token.setter
    def next_token(self, value: str) -> None:
        self._next_token = value

    @next_token.deleter
    def next_token(self) -> None:
        self._next_token = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..models.sample_group_status import SampleGroupStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="SampleGroupStatusUpdate")


@attr.s(auto_attribs=True, repr=False)
class SampleGroupStatusUpdate:
    """  """

    _sample_group_id: str
    _status: SampleGroupStatus
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("sample_group_id={}".format(repr(self._sample_group_id)))
        fields.append("status={}".format(repr(self._status)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "SampleGroupStatusUpdate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        sample_group_id = self._sample_group_id
        status = self._status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if sample_group_id is not UNSET:
            field_dict["sampleGroupId"] = sample_group_id
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_sample_group_id() -> str:
            sample_group_id = d.pop("sampleGroupId")
            return sample_group_id

        try:
            sample_group_id = get_sample_group_id()
        except KeyError:
            if strict:
                raise
            sample_group_id = cast(str, UNSET)

        def get_status() -> SampleGroupStatus:
            _status = d.pop("status")
            try:
                status = SampleGroupStatus(_status)
            except ValueError:
                status = SampleGroupStatus.of_unknown(_status)

            return status

        try:
            status = get_status()
        except KeyError:
            if strict:
                raise
            status = cast(SampleGroupStatus, UNSET)

        sample_group_status_update = cls(
            sample_group_id=sample_group_id,
            status=status,
        )

        sample_group_status_update.additional_properties = d
        return sample_group_status_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def sample_group_id(self) -> str:
        """ The string id of the sample group """
        if isinstance(self._sample_group_id, Unset):
            raise NotPresentError(self, "sample_group_id")
        return self._sample_group_id

    @sample_group_id.setter
    def sample_group_id(self, value: str) -> None:
        self._sample_group_id = value

    @property
    def status(self) -> SampleGroupStatus:
        """ Status of a sample group """
        if isinstance(self._status, Unset):
            raise NotPresentError(self, "status")
        return self._status

    @status.setter
    def status(self, value: SampleGroupStatus) -> None:
        self._status = value



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.bad_request_error_bulk_error_errors_item import BadRequestErrorBulkErrorErrorsItem
from ..types import UNSET, Unset

T = TypeVar("T", bound="BadRequestErrorBulkError")


@attr.s(auto_attribs=True, repr=False)
class BadRequestErrorBulkError:
    """  """

    _errors: Union[Unset, List[BadRequestErrorBulkErrorErrorsItem]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("errors={}".format(repr(self._errors)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "BadRequestErrorBulkError({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        errors: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._errors, Unset):
            errors = []
            for errors_item_data in self._errors:
                errors_item = errors_item_data.to_dict()

                errors.append(errors_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if errors is not UNSET:
            field_dict["errors"] = errors

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_errors() -> Union[Unset, List[BadRequestErrorBulkErrorErrorsItem]]:
            errors = []
            _errors = d.pop("errors")
            for errors_item_data in _errors or []:
                errors_item = BadRequestErrorBulkErrorErrorsItem.from_dict(errors_item_data, strict=False)

                errors.append(errors_item)

            return errors

        try:
            errors = get_errors()
        except KeyError:
            if strict:
                raise
            errors = cast(Union[Unset, List[BadRequestErrorBulkErrorErrorsItem]], UNSET)

        bad_request_error_bulk_error = cls(
            errors=errors,
        )

        bad_request_error_bulk_error.additional_properties = d
        return bad_request_error_bulk_error

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def errors(self) -> List[BadRequestErrorBulkErrorErrorsItem]:
        if isinstance(self._errors, Unset):
            raise NotPresentError(self, "errors")
        return self._errors

    @errors.setter
    def errors(self, value: List[BadRequestErrorBulkErrorErrorsItem]) -> None:
        self._errors = value

    @errors.deleter
    def errors(self) -> None:
        self._errors = UNSET



from typing import Any, cast, Dict, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..models.measurement import Measurement
from ..types import UNSET, Unset

T = TypeVar("T", bound="ContainerContentUpdate")


@attr.s(auto_attribs=True, repr=False)
class ContainerContentUpdate:
    """  """

    _concentration: Measurement

    def __repr__(self):
        fields = []
        fields.append("concentration={}".format(repr(self._concentration)))
        return "ContainerContentUpdate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        concentration = self._concentration.to_dict()

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if concentration is not UNSET:
            field_dict["concentration"] = concentration

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_concentration() -> Measurement:
            concentration = Measurement.from_dict(d.pop("concentration"), strict=False)

            return concentration

        try:
            concentration = get_concentration()
        except KeyError:
            if strict:
                raise
            concentration = cast(Measurement, UNSET)

        container_content_update = cls(
            concentration=concentration,
        )

        return container_content_update

    @property
    def concentration(self) -> Measurement:
        if isinstance(self._concentration, Unset):
            raise NotPresentError(self, "concentration")
        return self._concentration

    @concentration.setter
    def concentration(self, value: Measurement) -> None:
        self._concentration = value



import base64
from dataclasses import dataclass
from datetime import datetime, timedelta
import threading
from typing import Optional

import httpx

from benchling_api_client.v2.benchling_client import AuthorizationMethod
from benchling_api_client.v2.types import Response

MINIMUM_TOKEN_EXPIRY_BUFFER = 60


class Token:
    """Represents an OAuth2 token response model."""

    def __init__(self, access_token: str, refresh_time: datetime):
        """
        Initialize Token.

        :param access_token: The raw token value for authorizing with the API
        :param refresh_time: Calculated value off of token time-to-live for when a new token should be generated.
        """
        self.access_token = access_token
        self.refresh_time = refresh_time

    def valid(self) -> bool:
        """Return whether token is still valid for use or should be regenerated."""
        return datetime.now() < self.refresh_time

    @classmethod
    def from_token_response(cls, token_response):
        """
        Construct Token from deserializing token endpoint response.

        Deserializes response from token endpoint and calculates expiry time with buffer for when token should be
        regenerated.

        :param token_response: The response from an RFC6749 POST /token endpoint.
        """
        token_type: str = token_response.get("token_type")
        access_token: str = token_response.get("access_token")
        expires_in: float = token_response.get("expires_in")
        assert token_type == "Bearer"
        # Add in a buffer to safeguard against race conditions with token expiration.
        # Buffer is 10% of expires_in time, clamped between [1, MINIMUM_TOKEN_EXPIRY_BUFFER] seconds.
        refresh_delta = expires_in - max(1, min(MINIMUM_TOKEN_EXPIRY_BUFFER, expires_in * 0.1))
        refresh_time = datetime.now() + timedelta(seconds=refresh_delta)
        return cls(access_token, refresh_time)


@dataclass
class TokenVendError(Exception):
    def __init__(self, response: Response):
        self.content = response.content
        super(TokenVendError, self).__init__(f"{response.status_code}: {response.content}")


class ClientCredentialsOAuth2(AuthorizationMethod):
    """
    OAuth2 client credentials for authorization.

    Use in combination with the Benchling() client constructor to be authorized with OAuth2 client_credentials grant
    type.
    """

    _data_for_token_request = {
        "grant_type": "client_credentials",
    }

    def __init__(self, client_id: str, client_secret: str, token_url: str):
        """
        Initialize ClientCredentialsOAuth2.

        :param client_id: Client id in client_credentials grant type
        :param client_secret: Client secret in client_credentials grant type
        :param token_url: A fully-qualified URL pointing at the access token request endpoint such as
                          https://benchling.com/api/v2/token
        """
        self._token_url = token_url
        token_encoded = base64.b64encode(f"{client_id}:{client_secret}".encode())
        self._header_for_token_request = {
            "Content-Type": "application/x-www-form-urlencoded",
            "Authorization": f"Basic {token_encoded.decode()}",
        }
        self._token: Optional[Token] = None
        self._lock = threading.Lock()

    def vend_new_token(self):
        """Make RFC6749 request to token URL to generate a new bearer token for client credentials OAuth2 flow."""
        response: httpx.Response = httpx.post(
            self._token_url,
            data=ClientCredentialsOAuth2._data_for_token_request,
            headers=self._header_for_token_request,
        )

        if response.status_code == 200:
            as_json = response.json()
            self._token = Token.from_token_response(as_json)
        else:
            raise TokenVendError(response)

    def get_authorization_header(self) -> str:
        """
        Generate HTTP Authorization request header.

        If a token has not yet been requested or is close to its expiry time, a new token is requested.
        Otherwise, re-use existing valid token.
        """
        with self._lock:
            if self._token is None or not self._token.valid():
                self.vend_new_token()
        assert self._token is not None
        return f"Bearer {self._token.access_token}"



from typing import Any, cast, Dict, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorklistItemCreate")


@attr.s(auto_attribs=True, repr=False)
class WorklistItemCreate:
    """  """

    _item_id: str

    def __repr__(self):
        fields = []
        fields.append("item_id={}".format(repr(self._item_id)))
        return "WorklistItemCreate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        item_id = self._item_id

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if item_id is not UNSET:
            field_dict["itemId"] = item_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_item_id() -> str:
            item_id = d.pop("itemId")
            return item_id

        try:
            item_id = get_item_id()
        except KeyError:
            if strict:
                raise
            item_id = cast(str, UNSET)

        worklist_item_create = cls(
            item_id=item_id,
        )

        return worklist_item_create

    @property
    def item_id(self) -> str:
        """ The ID of the item to add to the worklist. """
        if isinstance(self._item_id, Unset):
            raise NotPresentError(self, "item_id")
        return self._item_id

    @item_id.setter
    def item_id(self, value: str) -> None:
        self._item_id = value



from typing import Any, cast, Dict, List, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..models.canvases_archive_reason import CanvasesArchiveReason
from ..types import UNSET, Unset

T = TypeVar("T", bound="CanvasesArchive")


@attr.s(auto_attribs=True, repr=False)
class CanvasesArchive:
    """  """

    _canvas_ids: List[str]
    _reason: CanvasesArchiveReason

    def __repr__(self):
        fields = []
        fields.append("canvas_ids={}".format(repr(self._canvas_ids)))
        fields.append("reason={}".format(repr(self._reason)))
        return "CanvasesArchive({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        canvas_ids = self._canvas_ids

        reason = self._reason.value

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if canvas_ids is not UNSET:
            field_dict["canvasIds"] = canvas_ids
        if reason is not UNSET:
            field_dict["reason"] = reason

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_canvas_ids() -> List[str]:
            canvas_ids = cast(List[str], d.pop("canvasIds"))

            return canvas_ids

        try:
            canvas_ids = get_canvas_ids()
        except KeyError:
            if strict:
                raise
            canvas_ids = cast(List[str], UNSET)

        def get_reason() -> CanvasesArchiveReason:
            _reason = d.pop("reason")
            try:
                reason = CanvasesArchiveReason(_reason)
            except ValueError:
                reason = CanvasesArchiveReason.of_unknown(_reason)

            return reason

        try:
            reason = get_reason()
        except KeyError:
            if strict:
                raise
            reason = cast(CanvasesArchiveReason, UNSET)

        canvases_archive = cls(
            canvas_ids=canvas_ids,
            reason=reason,
        )

        return canvases_archive

    @property
    def canvas_ids(self) -> List[str]:
        """ Array of canvas IDs """
        if isinstance(self._canvas_ids, Unset):
            raise NotPresentError(self, "canvas_ids")
        return self._canvas_ids

    @canvas_ids.setter
    def canvas_ids(self, value: List[str]) -> None:
        self._canvas_ids = value

    @property
    def reason(self) -> CanvasesArchiveReason:
        """ Reason that apps are being archived. Actual reason enum varies by tenant. """
        if isinstance(self._reason, Unset):
            raise NotPresentError(self, "reason")
        return self._reason

    @reason.setter
    def reason(self, value: CanvasesArchiveReason) -> None:
        self._reason = value



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.button_ui_block import ButtonUiBlock
from ..models.chip_ui_block import ChipUiBlock
from ..models.dropdown_ui_block import DropdownUiBlock
from ..models.markdown_ui_block import MarkdownUiBlock
from ..models.search_input_ui_block import SearchInputUiBlock
from ..models.section_ui_block import SectionUiBlock
from ..models.selector_input_ui_block import SelectorInputUiBlock
from ..models.text_input_ui_block import TextInputUiBlock
from ..models.text_label_ui_block import TextLabelUiBlock
from ..types import UNSET, Unset

T = TypeVar("T", bound="CanvasUpdate")


@attr.s(auto_attribs=True, repr=False)
class CanvasUpdate:
    """  """

    _blocks: List[
        Union[
            ButtonUiBlock,
            ChipUiBlock,
            DropdownUiBlock,
            MarkdownUiBlock,
            SearchInputUiBlock,
            SectionUiBlock,
            SelectorInputUiBlock,
            TextInputUiBlock,
            TextLabelUiBlock,
            UnknownType,
        ]
    ]
    _enabled: Union[Unset, bool] = UNSET
    _feature_id: Union[Unset, str] = UNSET
    _resource_id: Union[Unset, str] = UNSET
    _session_id: Union[Unset, None, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("blocks={}".format(repr(self._blocks)))
        fields.append("enabled={}".format(repr(self._enabled)))
        fields.append("feature_id={}".format(repr(self._feature_id)))
        fields.append("resource_id={}".format(repr(self._resource_id)))
        fields.append("session_id={}".format(repr(self._session_id)))
        return "CanvasUpdate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        blocks = []
        for blocks_item_data in self._blocks:
            if isinstance(blocks_item_data, UnknownType):
                blocks_item = blocks_item_data.value
            elif isinstance(blocks_item_data, ButtonUiBlock):
                blocks_item = blocks_item_data.to_dict()

            elif isinstance(blocks_item_data, ChipUiBlock):
                blocks_item = blocks_item_data.to_dict()

            elif isinstance(blocks_item_data, DropdownUiBlock):
                blocks_item = blocks_item_data.to_dict()

            elif isinstance(blocks_item_data, MarkdownUiBlock):
                blocks_item = blocks_item_data.to_dict()

            elif isinstance(blocks_item_data, SearchInputUiBlock):
                blocks_item = blocks_item_data.to_dict()

            elif isinstance(blocks_item_data, SectionUiBlock):
                blocks_item = blocks_item_data.to_dict()

            elif isinstance(blocks_item_data, SelectorInputUiBlock):
                blocks_item = blocks_item_data.to_dict()

            elif isinstance(blocks_item_data, TextInputUiBlock):
                blocks_item = blocks_item_data.to_dict()

            else:
                blocks_item = blocks_item_data.to_dict()

            blocks.append(blocks_item)

        enabled = self._enabled
        feature_id = self._feature_id
        resource_id = self._resource_id
        session_id = self._session_id

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if blocks is not UNSET:
            field_dict["blocks"] = blocks
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if feature_id is not UNSET:
            field_dict["featureId"] = feature_id
        if resource_id is not UNSET:
            field_dict["resourceId"] = resource_id
        if session_id is not UNSET:
            field_dict["sessionId"] = session_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_blocks() -> List[
            Union[
                ButtonUiBlock,
                ChipUiBlock,
                DropdownUiBlock,
                MarkdownUiBlock,
                SearchInputUiBlock,
                SectionUiBlock,
                SelectorInputUiBlock,
                TextInputUiBlock,
                TextLabelUiBlock,
                UnknownType,
            ]
        ]:
            blocks = []
            _blocks = d.pop("blocks")
            for blocks_item_data in _blocks:

                def _parse_blocks_item(
                    data: Union[Dict[str, Any]]
                ) -> Union[
                    ButtonUiBlock,
                    ChipUiBlock,
                    DropdownUiBlock,
                    MarkdownUiBlock,
                    SearchInputUiBlock,
                    SectionUiBlock,
                    SelectorInputUiBlock,
                    TextInputUiBlock,
                    TextLabelUiBlock,
                    UnknownType,
                ]:
                    blocks_item: Union[
                        ButtonUiBlock,
                        ChipUiBlock,
                        DropdownUiBlock,
                        MarkdownUiBlock,
                        SearchInputUiBlock,
                        SectionUiBlock,
                        SelectorInputUiBlock,
                        TextInputUiBlock,
                        TextLabelUiBlock,
                        UnknownType,
                    ]
                    discriminator_value: str = cast(str, data.get("type"))
                    if discriminator_value is not None:
                        if discriminator_value == "BUTTON":
                            blocks_item = ButtonUiBlock.from_dict(data, strict=False)

                            return blocks_item
                        if discriminator_value == "CHIP":
                            blocks_item = ChipUiBlock.from_dict(data, strict=False)

                            return blocks_item
                        if discriminator_value == "DROPDOWN":
                            blocks_item = DropdownUiBlock.from_dict(data, strict=False)

                            return blocks_item
                        if discriminator_value == "LABEL":
                            blocks_item = TextLabelUiBlock.from_dict(data, strict=False)

                            return blocks_item
                        if discriminator_value == "MARKDOWN":
                            blocks_item = MarkdownUiBlock.from_dict(data, strict=False)

                            return blocks_item
                        if discriminator_value == "SEARCH_INPUT":
                            blocks_item = SearchInputUiBlock.from_dict(data, strict=False)

                            return blocks_item
                        if discriminator_value == "SECTION":
                            blocks_item = SectionUiBlock.from_dict(data, strict=False)

                            return blocks_item
                        if discriminator_value == "SELECTOR_INPUT":
                            blocks_item = SelectorInputUiBlock.from_dict(data, strict=False)

                            return blocks_item
                        if discriminator_value == "TEXT_INPUT":
                            blocks_item = TextInputUiBlock.from_dict(data, strict=False)

                            return blocks_item

                        return UnknownType(value=data)
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        blocks_item = ButtonUiBlock.from_dict(data, strict=True)

                        return blocks_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        blocks_item = ChipUiBlock.from_dict(data, strict=True)

                        return blocks_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        blocks_item = DropdownUiBlock.from_dict(data, strict=True)

                        return blocks_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        blocks_item = MarkdownUiBlock.from_dict(data, strict=True)

                        return blocks_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        blocks_item = SearchInputUiBlock.from_dict(data, strict=True)

                        return blocks_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        blocks_item = SectionUiBlock.from_dict(data, strict=True)

                        return blocks_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        blocks_item = SelectorInputUiBlock.from_dict(data, strict=True)

                        return blocks_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        blocks_item = TextInputUiBlock.from_dict(data, strict=True)

                        return blocks_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        blocks_item = TextLabelUiBlock.from_dict(data, strict=True)

                        return blocks_item
                    except:  # noqa: E722
                        pass
                    return UnknownType(data)

                blocks_item = _parse_blocks_item(blocks_item_data)

                blocks.append(blocks_item)

            return blocks

        try:
            blocks = get_blocks()
        except KeyError:
            if strict:
                raise
            blocks = cast(
                List[
                    Union[
                        ButtonUiBlock,
                        ChipUiBlock,
                        DropdownUiBlock,
                        MarkdownUiBlock,
                        SearchInputUiBlock,
                        SectionUiBlock,
                        SelectorInputUiBlock,
                        TextInputUiBlock,
                        TextLabelUiBlock,
                        UnknownType,
                    ]
                ],
                UNSET,
            )

        def get_enabled() -> Union[Unset, bool]:
            enabled = d.pop("enabled")
            return enabled

        try:
            enabled = get_enabled()
        except KeyError:
            if strict:
                raise
            enabled = cast(Union[Unset, bool], UNSET)

        def get_feature_id() -> Union[Unset, str]:
            feature_id = d.pop("featureId")
            return feature_id

        try:
            feature_id = get_feature_id()
        except KeyError:
            if strict:
                raise
            feature_id = cast(Union[Unset, str], UNSET)

        def get_resource_id() -> Union[Unset, str]:
            resource_id = d.pop("resourceId")
            return resource_id

        try:
            resource_id = get_resource_id()
        except KeyError:
            if strict:
                raise
            resource_id = cast(Union[Unset, str], UNSET)

        def get_session_id() -> Union[Unset, None, str]:
            session_id = d.pop("sessionId")
            return session_id

        try:
            session_id = get_session_id()
        except KeyError:
            if strict:
                raise
            session_id = cast(Union[Unset, None, str], UNSET)

        canvas_update = cls(
            blocks=blocks,
            enabled=enabled,
            feature_id=feature_id,
            resource_id=resource_id,
            session_id=session_id,
        )

        return canvas_update

    @property
    def blocks(
        self,
    ) -> List[
        Union[
            ButtonUiBlock,
            ChipUiBlock,
            DropdownUiBlock,
            MarkdownUiBlock,
            SearchInputUiBlock,
            SectionUiBlock,
            SelectorInputUiBlock,
            TextInputUiBlock,
            TextLabelUiBlock,
            UnknownType,
        ]
    ]:
        if isinstance(self._blocks, Unset):
            raise NotPresentError(self, "blocks")
        return self._blocks

    @blocks.setter
    def blocks(
        self,
        value: List[
            Union[
                ButtonUiBlock,
                ChipUiBlock,
                DropdownUiBlock,
                MarkdownUiBlock,
                SearchInputUiBlock,
                SectionUiBlock,
                SelectorInputUiBlock,
                TextInputUiBlock,
                TextLabelUiBlock,
                UnknownType,
            ]
        ],
    ) -> None:
        self._blocks = value

    @property
    def enabled(self) -> bool:
        """Overall control for whether the canvas is interactable or not. If `false`, every block is disabled and will override the individual block's `enabled` property. If `true` or absent, the interactivity status will defer to the block's `enabled` property."""
        if isinstance(self._enabled, Unset):
            raise NotPresentError(self, "enabled")
        return self._enabled

    @enabled.setter
    def enabled(self, value: bool) -> None:
        self._enabled = value

    @enabled.deleter
    def enabled(self) -> None:
        self._enabled = UNSET

    @property
    def feature_id(self) -> str:
        """ Identifier of the feature defined in Benchling App Manifest this canvas corresponds to. """
        if isinstance(self._feature_id, Unset):
            raise NotPresentError(self, "feature_id")
        return self._feature_id

    @feature_id.setter
    def feature_id(self, value: str) -> None:
        self._feature_id = value

    @feature_id.deleter
    def feature_id(self) -> None:
        self._feature_id = UNSET

    @property
    def resource_id(self) -> str:
        """ Identifier of the resource object to attach canvas to. """
        if isinstance(self._resource_id, Unset):
            raise NotPresentError(self, "resource_id")
        return self._resource_id

    @resource_id.setter
    def resource_id(self, value: str) -> None:
        self._resource_id = value

    @resource_id.deleter
    def resource_id(self) -> None:
        self._resource_id = UNSET

    @property
    def session_id(self) -> Optional[str]:
        """Identifier of a session. If specified, app status messages from the session will be reported in the canvas."""
        if isinstance(self._session_id, Unset):
            raise NotPresentError(self, "session_id")
        return self._session_id

    @session_id.setter
    def session_id(self, value: Optional[str]) -> None:
        self._session_id = value

    @session_id.deleter
    def session_id(self) -> None:
        self._session_id = UNSET



from typing import Any, cast, Dict, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="FeatureUpdate")


@attr.s(auto_attribs=True, repr=False)
class FeatureUpdate:
    """ Inputs for updating a feature """

    _color: Union[Unset, str] = UNSET
    _feature_library_id: Union[Unset, str] = UNSET
    _feature_type: Union[Unset, None, str] = UNSET
    _name: Union[Unset, str] = UNSET
    _pattern: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("color={}".format(repr(self._color)))
        fields.append("feature_library_id={}".format(repr(self._feature_library_id)))
        fields.append("feature_type={}".format(repr(self._feature_type)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("pattern={}".format(repr(self._pattern)))
        return "FeatureUpdate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        color = self._color
        feature_library_id = self._feature_library_id
        feature_type = self._feature_type
        name = self._name
        pattern = self._pattern

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if color is not UNSET:
            field_dict["color"] = color
        if feature_library_id is not UNSET:
            field_dict["featureLibraryId"] = feature_library_id
        if feature_type is not UNSET:
            field_dict["featureType"] = feature_type
        if name is not UNSET:
            field_dict["name"] = name
        if pattern is not UNSET:
            field_dict["pattern"] = pattern

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_color() -> Union[Unset, str]:
            color = d.pop("color")
            return color

        try:
            color = get_color()
        except KeyError:
            if strict:
                raise
            color = cast(Union[Unset, str], UNSET)

        def get_feature_library_id() -> Union[Unset, str]:
            feature_library_id = d.pop("featureLibraryId")
            return feature_library_id

        try:
            feature_library_id = get_feature_library_id()
        except KeyError:
            if strict:
                raise
            feature_library_id = cast(Union[Unset, str], UNSET)

        def get_feature_type() -> Union[Unset, None, str]:
            feature_type = d.pop("featureType")
            return feature_type

        try:
            feature_type = get_feature_type()
        except KeyError:
            if strict:
                raise
            feature_type = cast(Union[Unset, None, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_pattern() -> Union[Unset, str]:
            pattern = d.pop("pattern")
            return pattern

        try:
            pattern = get_pattern()
        except KeyError:
            if strict:
                raise
            pattern = cast(Union[Unset, str], UNSET)

        feature_update = cls(
            color=color,
            feature_library_id=feature_library_id,
            feature_type=feature_type,
            name=name,
            pattern=pattern,
        )

        return feature_update

    @property
    def color(self) -> str:
        """ The color of the annotations generated by the feature. Must be a valid hex string """
        if isinstance(self._color, Unset):
            raise NotPresentError(self, "color")
        return self._color

    @color.setter
    def color(self, value: str) -> None:
        self._color = value

    @color.deleter
    def color(self) -> None:
        self._color = UNSET

    @property
    def feature_library_id(self) -> str:
        """ The id of the feature library the feature belongs to """
        if isinstance(self._feature_library_id, Unset):
            raise NotPresentError(self, "feature_library_id")
        return self._feature_library_id

    @feature_library_id.setter
    def feature_library_id(self, value: str) -> None:
        self._feature_library_id = value

    @feature_library_id.deleter
    def feature_library_id(self) -> None:
        self._feature_library_id = UNSET

    @property
    def feature_type(self) -> Optional[str]:
        """The type of feature, like gene, promoter, etc. Note: This is an arbitrary string, not an enum"""
        if isinstance(self._feature_type, Unset):
            raise NotPresentError(self, "feature_type")
        return self._feature_type

    @feature_type.setter
    def feature_type(self, value: Optional[str]) -> None:
        self._feature_type = value

    @feature_type.deleter
    def feature_type(self) -> None:
        self._feature_type = UNSET

    @property
    def name(self) -> str:
        """ The name of the feature """
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def pattern(self) -> str:
        """ The pattern used for matching during auto-annotation. """
        if isinstance(self._pattern, Unset):
            raise NotPresentError(self, "pattern")
        return self._pattern

    @pattern.setter
    def pattern(self, value: str) -> None:
        self._pattern = value

    @pattern.deleter
    def pattern(self) -> None:
        self._pattern = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..models.assay_run_create import AssayRunCreate
from ..types import UNSET, Unset

T = TypeVar("T", bound="AssayRunsBulkCreateRequest")


@attr.s(auto_attribs=True, repr=False)
class AssayRunsBulkCreateRequest:
    """  """

    _assay_runs: List[AssayRunCreate]

    def __repr__(self):
        fields = []
        fields.append("assay_runs={}".format(repr(self._assay_runs)))
        return "AssayRunsBulkCreateRequest({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        assay_runs = []
        for assay_runs_item_data in self._assay_runs:
            assay_runs_item = assay_runs_item_data.to_dict()

            assay_runs.append(assay_runs_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if assay_runs is not UNSET:
            field_dict["assayRuns"] = assay_runs

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_assay_runs() -> List[AssayRunCreate]:
            assay_runs = []
            _assay_runs = d.pop("assayRuns")
            for assay_runs_item_data in _assay_runs:
                assay_runs_item = AssayRunCreate.from_dict(assay_runs_item_data, strict=False)

                assay_runs.append(assay_runs_item)

            return assay_runs

        try:
            assay_runs = get_assay_runs()
        except KeyError:
            if strict:
                raise
            assay_runs = cast(List[AssayRunCreate], UNSET)

        assay_runs_bulk_create_request = cls(
            assay_runs=assay_runs,
        )

        return assay_runs_bulk_create_request

    @property
    def assay_runs(self) -> List[AssayRunCreate]:
        if isinstance(self._assay_runs, Unset):
            raise NotPresentError(self, "assay_runs")
        return self._assay_runs

    @assay_runs.setter
    def assay_runs(self, value: List[AssayRunCreate]) -> None:
        self._assay_runs = value



"""Contains some shared types for properties."""
from __future__ import annotations

from typing import BinaryIO, Generic, MutableMapping, Optional, TextIO, Tuple, TypeVar, Union

import attr


class Unset:
    def __bool__(self) -> bool:
        return False

    def __copy__(self) -> Unset:
        return UNSET

    def __deepcopy__(self, memo) -> Unset:
        return UNSET


UNSET: Unset = Unset()

# Used as `FileProperty._json_type_string`
FileJsonType = Tuple[Optional[str], Union[BinaryIO, TextIO], Optional[str]]


@attr.s(auto_attribs=True)
class File:
    """Contains information for file uploads."""

    payload: Union[BinaryIO, TextIO]
    file_name: Optional[str] = None
    mime_type: Optional[str] = None

    def to_tuple(self) -> FileJsonType:
        """Return a tuple representation that httpx will accept for multipart/form-data."""
        return self.file_name, self.payload, self.mime_type


T = TypeVar("T")


@attr.s(auto_attribs=True)
class Response(Generic[T]):
    """A response from an endpoint."""

    status_code: int
    content: bytes
    headers: MutableMapping[str, str]
    parsed: Optional[T]


__all__ = ["File", "Response"]



from typing import Any, cast, Dict, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="AutomationOutputProcessorCreate")


@attr.s(auto_attribs=True, repr=False)
class AutomationOutputProcessorCreate:
    """  """

    _assay_run_id: str
    _automation_file_config_name: str
    _file_id: str
    _complete_with_errors: Union[Unset, bool] = UNSET

    def __repr__(self):
        fields = []
        fields.append("assay_run_id={}".format(repr(self._assay_run_id)))
        fields.append("automation_file_config_name={}".format(repr(self._automation_file_config_name)))
        fields.append("file_id={}".format(repr(self._file_id)))
        fields.append("complete_with_errors={}".format(repr(self._complete_with_errors)))
        return "AutomationOutputProcessorCreate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        assay_run_id = self._assay_run_id
        automation_file_config_name = self._automation_file_config_name
        file_id = self._file_id
        complete_with_errors = self._complete_with_errors

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if assay_run_id is not UNSET:
            field_dict["assayRunId"] = assay_run_id
        if automation_file_config_name is not UNSET:
            field_dict["automationFileConfigName"] = automation_file_config_name
        if file_id is not UNSET:
            field_dict["fileId"] = file_id
        if complete_with_errors is not UNSET:
            field_dict["completeWithErrors"] = complete_with_errors

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_assay_run_id() -> str:
            assay_run_id = d.pop("assayRunId")
            return assay_run_id

        try:
            assay_run_id = get_assay_run_id()
        except KeyError:
            if strict:
                raise
            assay_run_id = cast(str, UNSET)

        def get_automation_file_config_name() -> str:
            automation_file_config_name = d.pop("automationFileConfigName")
            return automation_file_config_name

        try:
            automation_file_config_name = get_automation_file_config_name()
        except KeyError:
            if strict:
                raise
            automation_file_config_name = cast(str, UNSET)

        def get_file_id() -> str:
            file_id = d.pop("fileId")
            return file_id

        try:
            file_id = get_file_id()
        except KeyError:
            if strict:
                raise
            file_id = cast(str, UNSET)

        def get_complete_with_errors() -> Union[Unset, bool]:
            complete_with_errors = d.pop("completeWithErrors")
            return complete_with_errors

        try:
            complete_with_errors = get_complete_with_errors()
        except KeyError:
            if strict:
                raise
            complete_with_errors = cast(Union[Unset, bool], UNSET)

        automation_output_processor_create = cls(
            assay_run_id=assay_run_id,
            automation_file_config_name=automation_file_config_name,
            file_id=file_id,
            complete_with_errors=complete_with_errors,
        )

        return automation_output_processor_create

    @property
    def assay_run_id(self) -> str:
        if isinstance(self._assay_run_id, Unset):
            raise NotPresentError(self, "assay_run_id")
        return self._assay_run_id

    @assay_run_id.setter
    def assay_run_id(self, value: str) -> None:
        self._assay_run_id = value

    @property
    def automation_file_config_name(self) -> str:
        if isinstance(self._automation_file_config_name, Unset):
            raise NotPresentError(self, "automation_file_config_name")
        return self._automation_file_config_name

    @automation_file_config_name.setter
    def automation_file_config_name(self, value: str) -> None:
        self._automation_file_config_name = value

    @property
    def file_id(self) -> str:
        """ The ID of a blob link to process. """
        if isinstance(self._file_id, Unset):
            raise NotPresentError(self, "file_id")
        return self._file_id

    @file_id.setter
    def file_id(self, value: str) -> None:
        self._file_id = value

    @property
    def complete_with_errors(self) -> bool:
        """ Specifies whether file processing should complete with errors. False means any error in output file processing will result in no actions being committed. True means that if row-level errors occur, then failing rows and their errors will be saved to errorFile, and actions from successful rows will be committed. """
        if isinstance(self._complete_with_errors, Unset):
            raise NotPresentError(self, "complete_with_errors")
        return self._complete_with_errors

    @complete_with_errors.setter
    def complete_with_errors(self, value: bool) -> None:
        self._complete_with_errors = value

    @complete_with_errors.deleter
    def complete_with_errors(self) -> None:
        self._complete_with_errors = UNSET



from typing import Any, Dict, List, Optional, Type, TypeVar

import attr

T = TypeVar("T", bound="ExecuteSampleGroups")


@attr.s(auto_attribs=True, repr=False)
class ExecuteSampleGroups:
    """The response is intentionally empty."""

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "ExecuteSampleGroups({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        execute_sample_groups = cls()

        execute_sample_groups.additional_properties = d
        return execute_sample_groups

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="AaAnnotation")


@attr.s(auto_attribs=True, repr=False)
class AaAnnotation:
    """  """

    _color: Union[Unset, str] = UNSET
    _end: Union[Unset, int] = UNSET
    _id: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET
    _start: Union[Unset, int] = UNSET
    _type: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("color={}".format(repr(self._color)))
        fields.append("end={}".format(repr(self._end)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("start={}".format(repr(self._start)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "AaAnnotation({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        color = self._color
        end = self._end
        id = self._id
        name = self._name
        start = self._start
        type = self._type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if color is not UNSET:
            field_dict["color"] = color
        if end is not UNSET:
            field_dict["end"] = end
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if start is not UNSET:
            field_dict["start"] = start
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_color() -> Union[Unset, str]:
            color = d.pop("color")
            return color

        try:
            color = get_color()
        except KeyError:
            if strict:
                raise
            color = cast(Union[Unset, str], UNSET)

        def get_end() -> Union[Unset, int]:
            end = d.pop("end")
            return end

        try:
            end = get_end()
        except KeyError:
            if strict:
                raise
            end = cast(Union[Unset, int], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_start() -> Union[Unset, int]:
            start = d.pop("start")
            return start

        try:
            start = get_start()
        except KeyError:
            if strict:
                raise
            start = cast(Union[Unset, int], UNSET)

        def get_type() -> Union[Unset, str]:
            type = d.pop("type")
            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, str], UNSET)

        aa_annotation = cls(
            color=color,
            end=end,
            id=id,
            name=name,
            start=start,
            type=type,
        )

        aa_annotation.additional_properties = d
        return aa_annotation

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def color(self) -> str:
        if isinstance(self._color, Unset):
            raise NotPresentError(self, "color")
        return self._color

    @color.setter
    def color(self, value: str) -> None:
        self._color = value

    @color.deleter
    def color(self) -> None:
        self._color = UNSET

    @property
    def end(self) -> int:
        if isinstance(self._end, Unset):
            raise NotPresentError(self, "end")
        return self._end

    @end.setter
    def end(self, value: int) -> None:
        self._end = value

    @end.deleter
    def end(self) -> None:
        self._end = UNSET

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def start(self) -> int:
        if isinstance(self._start, Unset):
            raise NotPresentError(self, "start")
        return self._start

    @start.setter
    def start(self, value: int) -> None:
        self._start = value

    @start.deleter
    def start(self) -> None:
        self._start = UNSET

    @property
    def type(self) -> str:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: str) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="RequestWriteTeamAssignee")


@attr.s(auto_attribs=True, repr=False)
class RequestWriteTeamAssignee:
    """  """

    _team_id: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("team_id={}".format(repr(self._team_id)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "RequestWriteTeamAssignee({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        team_id = self._team_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if team_id is not UNSET:
            field_dict["teamId"] = team_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_team_id() -> str:
            team_id = d.pop("teamId")
            return team_id

        try:
            team_id = get_team_id()
        except KeyError:
            if strict:
                raise
            team_id = cast(str, UNSET)

        request_write_team_assignee = cls(
            team_id=team_id,
        )

        request_write_team_assignee.additional_properties = d
        return request_write_team_assignee

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def team_id(self) -> str:
        if isinstance(self._team_id, Unset):
            raise NotPresentError(self, "team_id")
        return self._team_id

    @team_id.setter
    def team_id(self, value: str) -> None:
        self._team_id = value



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.session_message_create import SessionMessageCreate
from ..types import UNSET, Unset

T = TypeVar("T", bound="SessionCreate")


@attr.s(auto_attribs=True, repr=False)
class SessionCreate:
    """  """

    _app_id: str
    _name: str
    _timeout_seconds: int
    _messages: Union[Unset, List[SessionMessageCreate]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("app_id={}".format(repr(self._app_id)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("timeout_seconds={}".format(repr(self._timeout_seconds)))
        fields.append("messages={}".format(repr(self._messages)))
        return "SessionCreate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        app_id = self._app_id
        name = self._name
        timeout_seconds = self._timeout_seconds
        messages: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._messages, Unset):
            messages = []
            for messages_item_data in self._messages:
                messages_item = messages_item_data.to_dict()

                messages.append(messages_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if app_id is not UNSET:
            field_dict["appId"] = app_id
        if name is not UNSET:
            field_dict["name"] = name
        if timeout_seconds is not UNSET:
            field_dict["timeoutSeconds"] = timeout_seconds
        if messages is not UNSET:
            field_dict["messages"] = messages

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_app_id() -> str:
            app_id = d.pop("appId")
            return app_id

        try:
            app_id = get_app_id()
        except KeyError:
            if strict:
                raise
            app_id = cast(str, UNSET)

        def get_name() -> str:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(str, UNSET)

        def get_timeout_seconds() -> int:
            timeout_seconds = d.pop("timeoutSeconds")
            return timeout_seconds

        try:
            timeout_seconds = get_timeout_seconds()
        except KeyError:
            if strict:
                raise
            timeout_seconds = cast(int, UNSET)

        def get_messages() -> Union[Unset, List[SessionMessageCreate]]:
            messages = []
            _messages = d.pop("messages")
            for messages_item_data in _messages or []:
                messages_item = SessionMessageCreate.from_dict(messages_item_data, strict=False)

                messages.append(messages_item)

            return messages

        try:
            messages = get_messages()
        except KeyError:
            if strict:
                raise
            messages = cast(Union[Unset, List[SessionMessageCreate]], UNSET)

        session_create = cls(
            app_id=app_id,
            name=name,
            timeout_seconds=timeout_seconds,
            messages=messages,
        )

        return session_create

    @property
    def app_id(self) -> str:
        if isinstance(self._app_id, Unset):
            raise NotPresentError(self, "app_id")
        return self._app_id

    @app_id.setter
    def app_id(self, value: str) -> None:
        self._app_id = value

    @property
    def name(self) -> str:
        """ The name of the session. Length must be between 3-100 chars. Value is required and immutable once set. """
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @property
    def timeout_seconds(self) -> int:
        """Timeout in seconds, a value between 1 second and 30 days. Once set, it can only be increased, not decreased."""
        if isinstance(self._timeout_seconds, Unset):
            raise NotPresentError(self, "timeout_seconds")
        return self._timeout_seconds

    @timeout_seconds.setter
    def timeout_seconds(self, value: int) -> None:
        self._timeout_seconds = value

    @property
    def messages(self) -> List[SessionMessageCreate]:
        """An array of `SessionMessage` describing the current session state."""
        if isinstance(self._messages, Unset):
            raise NotPresentError(self, "messages")
        return self._messages

    @messages.setter
    def messages(self, value: List[SessionMessageCreate]) -> None:
        self._messages = value

    @messages.deleter
    def messages(self) -> None:
        self._messages = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.batch_schema import BatchSchema
from ..types import UNSET, Unset

T = TypeVar("T", bound="BatchSchemasList")


@attr.s(auto_attribs=True, repr=False)
class BatchSchemasList:
    """  """

    _batch_schemas: Union[Unset, List[BatchSchema]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("batch_schemas={}".format(repr(self._batch_schemas)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "BatchSchemasList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        batch_schemas: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._batch_schemas, Unset):
            batch_schemas = []
            for batch_schemas_item_data in self._batch_schemas:
                batch_schemas_item = batch_schemas_item_data.to_dict()

                batch_schemas.append(batch_schemas_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if batch_schemas is not UNSET:
            field_dict["batchSchemas"] = batch_schemas

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_batch_schemas() -> Union[Unset, List[BatchSchema]]:
            batch_schemas = []
            _batch_schemas = d.pop("batchSchemas")
            for batch_schemas_item_data in _batch_schemas or []:
                batch_schemas_item = BatchSchema.from_dict(batch_schemas_item_data, strict=False)

                batch_schemas.append(batch_schemas_item)

            return batch_schemas

        try:
            batch_schemas = get_batch_schemas()
        except KeyError:
            if strict:
                raise
            batch_schemas = cast(Union[Unset, List[BatchSchema]], UNSET)

        batch_schemas_list = cls(
            batch_schemas=batch_schemas,
        )

        batch_schemas_list.additional_properties = d
        return batch_schemas_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def batch_schemas(self) -> List[BatchSchema]:
        if isinstance(self._batch_schemas, Unset):
            raise NotPresentError(self, "batch_schemas")
        return self._batch_schemas

    @batch_schemas.setter
    def batch_schemas(self, value: List[BatchSchema]) -> None:
        self._batch_schemas = value

    @batch_schemas.deleter
    def batch_schemas(self) -> None:
        self._batch_schemas = UNSET



from typing import Union

from ..extensions import UnknownType
from ..models.app_config_item_boolean_update import AppConfigItemBooleanUpdate
from ..models.app_config_item_date_update import AppConfigItemDateUpdate
from ..models.app_config_item_datetime_update import AppConfigItemDatetimeUpdate
from ..models.app_config_item_float_update import AppConfigItemFloatUpdate
from ..models.app_config_item_generic_update import AppConfigItemGenericUpdate
from ..models.app_config_item_integer_update import AppConfigItemIntegerUpdate
from ..models.app_config_item_json_update import AppConfigItemJsonUpdate

AppConfigItemUpdate = Union[
    AppConfigItemGenericUpdate,
    AppConfigItemBooleanUpdate,
    AppConfigItemIntegerUpdate,
    AppConfigItemFloatUpdate,
    AppConfigItemDateUpdate,
    AppConfigItemDatetimeUpdate,
    AppConfigItemJsonUpdate,
    UnknownType,
]



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.file_update_upload_status import FileUpdateUploadStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="FileUpdate")


@attr.s(auto_attribs=True, repr=False)
class FileUpdate:
    """  """

    _upload_status: Union[Unset, FileUpdateUploadStatus] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("upload_status={}".format(repr(self._upload_status)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "FileUpdate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        upload_status: Union[Unset, int] = UNSET
        if not isinstance(self._upload_status, Unset):
            upload_status = self._upload_status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if upload_status is not UNSET:
            field_dict["uploadStatus"] = upload_status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_upload_status() -> Union[Unset, FileUpdateUploadStatus]:
            upload_status = UNSET
            _upload_status = d.pop("uploadStatus")
            if _upload_status is not None and _upload_status is not UNSET:
                try:
                    upload_status = FileUpdateUploadStatus(_upload_status)
                except ValueError:
                    upload_status = FileUpdateUploadStatus.of_unknown(_upload_status)

            return upload_status

        try:
            upload_status = get_upload_status()
        except KeyError:
            if strict:
                raise
            upload_status = cast(Union[Unset, FileUpdateUploadStatus], UNSET)

        file_update = cls(
            upload_status=upload_status,
        )

        file_update.additional_properties = d
        return file_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def upload_status(self) -> FileUpdateUploadStatus:
        if isinstance(self._upload_status, Unset):
            raise NotPresentError(self, "upload_status")
        return self._upload_status

    @upload_status.setter
    def upload_status(self, value: FileUpdateUploadStatus) -> None:
        self._upload_status = value

    @upload_status.deleter
    def upload_status(self) -> None:
        self._upload_status = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..models.app_config_item_generic_create_type import AppConfigItemGenericCreateType
from ..types import UNSET, Unset

T = TypeVar("T", bound="AppConfigItemGenericCreate")


@attr.s(auto_attribs=True, repr=False)
class AppConfigItemGenericCreate:
    """  """

    _type: AppConfigItemGenericCreateType
    _value: str
    _app_id: str
    _path: List[str]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("type={}".format(repr(self._type)))
        fields.append("value={}".format(repr(self._value)))
        fields.append("app_id={}".format(repr(self._app_id)))
        fields.append("path={}".format(repr(self._path)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "AppConfigItemGenericCreate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        type = self._type.value

        value = self._value
        app_id = self._app_id
        path = self._path

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if type is not UNSET:
            field_dict["type"] = type
        if value is not UNSET:
            field_dict["value"] = value
        if app_id is not UNSET:
            field_dict["appId"] = app_id
        if path is not UNSET:
            field_dict["path"] = path

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_type() -> AppConfigItemGenericCreateType:
            _type = d.pop("type")
            try:
                type = AppConfigItemGenericCreateType(_type)
            except ValueError:
                type = AppConfigItemGenericCreateType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(AppConfigItemGenericCreateType, UNSET)

        def get_value() -> str:
            value = d.pop("value")
            return value

        try:
            value = get_value()
        except KeyError:
            if strict:
                raise
            value = cast(str, UNSET)

        def get_app_id() -> str:
            app_id = d.pop("appId")
            return app_id

        try:
            app_id = get_app_id()
        except KeyError:
            if strict:
                raise
            app_id = cast(str, UNSET)

        def get_path() -> List[str]:
            path = cast(List[str], d.pop("path"))

            return path

        try:
            path = get_path()
        except KeyError:
            if strict:
                raise
            path = cast(List[str], UNSET)

        app_config_item_generic_create = cls(
            type=type,
            value=value,
            app_id=app_id,
            path=path,
        )

        app_config_item_generic_create.additional_properties = d
        return app_config_item_generic_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def type(self) -> AppConfigItemGenericCreateType:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: AppConfigItemGenericCreateType) -> None:
        self._type = value

    @property
    def value(self) -> str:
        if isinstance(self._value, Unset):
            raise NotPresentError(self, "value")
        return self._value

    @value.setter
    def value(self, value: str) -> None:
        self._value = value

    @property
    def app_id(self) -> str:
        """ App id to which this config item belongs. """
        if isinstance(self._app_id, Unset):
            raise NotPresentError(self, "app_id")
        return self._app_id

    @app_id.setter
    def app_id(self, value: str) -> None:
        self._app_id = value

    @property
    def path(self) -> List[str]:
        """ Array-based representation of config item's location in the tree in order from top to bottom. """
        if isinstance(self._path, Unset):
            raise NotPresentError(self, "path")
        return self._path

    @path.setter
    def path(self, value: List[str]) -> None:
        self._path = value



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.printer import Printer
from ..types import UNSET, Unset

T = TypeVar("T", bound="PrintersList")


@attr.s(auto_attribs=True, repr=False)
class PrintersList:
    """  """

    _label_printers: Union[Unset, List[Printer]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("label_printers={}".format(repr(self._label_printers)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "PrintersList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        label_printers: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._label_printers, Unset):
            label_printers = []
            for label_printers_item_data in self._label_printers:
                label_printers_item = label_printers_item_data.to_dict()

                label_printers.append(label_printers_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if label_printers is not UNSET:
            field_dict["labelPrinters"] = label_printers

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_label_printers() -> Union[Unset, List[Printer]]:
            label_printers = []
            _label_printers = d.pop("labelPrinters")
            for label_printers_item_data in _label_printers or []:
                label_printers_item = Printer.from_dict(label_printers_item_data, strict=False)

                label_printers.append(label_printers_item)

            return label_printers

        try:
            label_printers = get_label_printers()
        except KeyError:
            if strict:
                raise
            label_printers = cast(Union[Unset, List[Printer]], UNSET)

        printers_list = cls(
            label_printers=label_printers,
        )

        printers_list.additional_properties = d
        return printers_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def label_printers(self) -> List[Printer]:
        if isinstance(self._label_printers, Unset):
            raise NotPresentError(self, "label_printers")
        return self._label_printers

    @label_printers.setter
    def label_printers(self, value: List[Printer]) -> None:
        self._label_printers = value

    @label_printers.deleter
    def label_printers(self) -> None:
        self._label_printers = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..models.canvas_interaction_webhook_type import CanvasInteractionWebhookType
from ..types import UNSET, Unset

T = TypeVar("T", bound="CanvasInteractionWebhook")


@attr.s(auto_attribs=True, repr=False)
class CanvasInteractionWebhook:
    """ Sent when a user interacts with an app canvas via button press """

    _button_id: str
    _canvas_id: str
    _type: CanvasInteractionWebhookType
    _deprecated: bool
    _excluded_properties: List[str]

    def __repr__(self):
        fields = []
        fields.append("button_id={}".format(repr(self._button_id)))
        fields.append("canvas_id={}".format(repr(self._canvas_id)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("deprecated={}".format(repr(self._deprecated)))
        fields.append("excluded_properties={}".format(repr(self._excluded_properties)))
        return "CanvasInteractionWebhook({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        button_id = self._button_id
        canvas_id = self._canvas_id
        type = self._type.value

        deprecated = self._deprecated
        excluded_properties = self._excluded_properties

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if button_id is not UNSET:
            field_dict["buttonId"] = button_id
        if canvas_id is not UNSET:
            field_dict["canvasId"] = canvas_id
        if type is not UNSET:
            field_dict["type"] = type
        if deprecated is not UNSET:
            field_dict["deprecated"] = deprecated
        if excluded_properties is not UNSET:
            field_dict["excludedProperties"] = excluded_properties

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_button_id() -> str:
            button_id = d.pop("buttonId")
            return button_id

        try:
            button_id = get_button_id()
        except KeyError:
            if strict:
                raise
            button_id = cast(str, UNSET)

        def get_canvas_id() -> str:
            canvas_id = d.pop("canvasId")
            return canvas_id

        try:
            canvas_id = get_canvas_id()
        except KeyError:
            if strict:
                raise
            canvas_id = cast(str, UNSET)

        def get_type() -> CanvasInteractionWebhookType:
            _type = d.pop("type")
            try:
                type = CanvasInteractionWebhookType(_type)
            except ValueError:
                type = CanvasInteractionWebhookType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(CanvasInteractionWebhookType, UNSET)

        def get_deprecated() -> bool:
            deprecated = d.pop("deprecated")
            return deprecated

        try:
            deprecated = get_deprecated()
        except KeyError:
            if strict:
                raise
            deprecated = cast(bool, UNSET)

        def get_excluded_properties() -> List[str]:
            excluded_properties = cast(List[str], d.pop("excludedProperties"))

            return excluded_properties

        try:
            excluded_properties = get_excluded_properties()
        except KeyError:
            if strict:
                raise
            excluded_properties = cast(List[str], UNSET)

        canvas_interaction_webhook = cls(
            button_id=button_id,
            canvas_id=canvas_id,
            type=type,
            deprecated=deprecated,
            excluded_properties=excluded_properties,
        )

        return canvas_interaction_webhook

    @property
    def button_id(self) -> str:
        if isinstance(self._button_id, Unset):
            raise NotPresentError(self, "button_id")
        return self._button_id

    @button_id.setter
    def button_id(self, value: str) -> None:
        self._button_id = value

    @property
    def canvas_id(self) -> str:
        if isinstance(self._canvas_id, Unset):
            raise NotPresentError(self, "canvas_id")
        return self._canvas_id

    @canvas_id.setter
    def canvas_id(self, value: str) -> None:
        self._canvas_id = value

    @property
    def type(self) -> CanvasInteractionWebhookType:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: CanvasInteractionWebhookType) -> None:
        self._type = value

    @property
    def deprecated(self) -> bool:
        if isinstance(self._deprecated, Unset):
            raise NotPresentError(self, "deprecated")
        return self._deprecated

    @deprecated.setter
    def deprecated(self, value: bool) -> None:
        self._deprecated = value

    @property
    def excluded_properties(self) -> List[str]:
        """These properties have been dropped from the payload due to size."""
        if isinstance(self._excluded_properties, Unset):
            raise NotPresentError(self, "excluded_properties")
        return self._excluded_properties

    @excluded_properties.setter
    def excluded_properties(self, value: List[str]) -> None:
        self._excluded_properties = value



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="MoleculesArchivalChange")


@attr.s(auto_attribs=True, repr=False)
class MoleculesArchivalChange:
    """IDs of all items that were archived or unarchived, grouped by resource type. This includes the IDs of Molecules along with any IDs of batches that were archived / unarchived."""

    _batch_ids: Union[Unset, List[str]] = UNSET
    _molecule_ids: Union[Unset, List[str]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("batch_ids={}".format(repr(self._batch_ids)))
        fields.append("molecule_ids={}".format(repr(self._molecule_ids)))
        return "MoleculesArchivalChange({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        batch_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._batch_ids, Unset):
            batch_ids = self._batch_ids

        molecule_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._molecule_ids, Unset):
            molecule_ids = self._molecule_ids

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if batch_ids is not UNSET:
            field_dict["batchIds"] = batch_ids
        if molecule_ids is not UNSET:
            field_dict["moleculeIds"] = molecule_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_batch_ids() -> Union[Unset, List[str]]:
            batch_ids = cast(List[str], d.pop("batchIds"))

            return batch_ids

        try:
            batch_ids = get_batch_ids()
        except KeyError:
            if strict:
                raise
            batch_ids = cast(Union[Unset, List[str]], UNSET)

        def get_molecule_ids() -> Union[Unset, List[str]]:
            molecule_ids = cast(List[str], d.pop("moleculeIds"))

            return molecule_ids

        try:
            molecule_ids = get_molecule_ids()
        except KeyError:
            if strict:
                raise
            molecule_ids = cast(Union[Unset, List[str]], UNSET)

        molecules_archival_change = cls(
            batch_ids=batch_ids,
            molecule_ids=molecule_ids,
        )

        return molecules_archival_change

    @property
    def batch_ids(self) -> List[str]:
        if isinstance(self._batch_ids, Unset):
            raise NotPresentError(self, "batch_ids")
        return self._batch_ids

    @batch_ids.setter
    def batch_ids(self, value: List[str]) -> None:
        self._batch_ids = value

    @batch_ids.deleter
    def batch_ids(self) -> None:
        self._batch_ids = UNSET

    @property
    def molecule_ids(self) -> List[str]:
        if isinstance(self._molecule_ids, Unset):
            raise NotPresentError(self, "molecule_ids")
        return self._molecule_ids

    @molecule_ids.setter
    def molecule_ids(self, value: List[str]) -> None:
        self._molecule_ids = value

    @molecule_ids.deleter
    def molecule_ids(self) -> None:
        self._molecule_ids = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="BaseNotePart")


@attr.s(auto_attribs=True, repr=False)
class BaseNotePart:
    """  """

    _indentation: Union[Unset, int] = 0
    _type: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("indentation={}".format(repr(self._indentation)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "BaseNotePart({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        indentation = self._indentation
        type = self._type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if indentation is not UNSET:
            field_dict["indentation"] = indentation
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_indentation() -> Union[Unset, int]:
            indentation = d.pop("indentation")
            return indentation

        try:
            indentation = get_indentation()
        except KeyError:
            if strict:
                raise
            indentation = cast(Union[Unset, int], UNSET)

        def get_type() -> Union[Unset, str]:
            type = d.pop("type")
            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, str], UNSET)

        base_note_part = cls(
            indentation=indentation,
            type=type,
        )

        base_note_part.additional_properties = d
        return base_note_part

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def indentation(self) -> int:
        """All notes have an indentation level - the default is 0 for no indent. For lists, indentation gives notes hierarchy - a bulleted list with children is modeled as one note part with indentation 1 followed by note parts with indentation 2, for example."""
        if isinstance(self._indentation, Unset):
            raise NotPresentError(self, "indentation")
        return self._indentation

    @indentation.setter
    def indentation(self, value: int) -> None:
        self._indentation = value

    @indentation.deleter
    def indentation(self) -> None:
        self._indentation = UNSET

    @property
    def type(self) -> str:
        """The type of the note.  Type determines what other fields are present."""
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: str) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.sequence_feature_custom_field import SequenceFeatureCustomField
from ..types import UNSET, Unset

T = TypeVar("T", bound="RnaAnnotation")


@attr.s(auto_attribs=True, repr=False)
class RnaAnnotation:
    """  """

    _end: Union[Unset, int] = UNSET
    _start: Union[Unset, int] = UNSET
    _strand: Union[Unset, int] = UNSET
    _type: Union[Unset, str] = UNSET
    _color: Union[Unset, str] = UNSET
    _custom_fields: Union[Unset, List[SequenceFeatureCustomField]] = UNSET
    _name: Union[Unset, str] = UNSET
    _notes: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("end={}".format(repr(self._end)))
        fields.append("start={}".format(repr(self._start)))
        fields.append("strand={}".format(repr(self._strand)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("color={}".format(repr(self._color)))
        fields.append("custom_fields={}".format(repr(self._custom_fields)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("notes={}".format(repr(self._notes)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "RnaAnnotation({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        end = self._end
        start = self._start
        strand = self._strand
        type = self._type
        color = self._color
        custom_fields: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._custom_fields, Unset):
            custom_fields = []
            for custom_fields_item_data in self._custom_fields:
                custom_fields_item = custom_fields_item_data.to_dict()

                custom_fields.append(custom_fields_item)

        name = self._name
        notes = self._notes

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if end is not UNSET:
            field_dict["end"] = end
        if start is not UNSET:
            field_dict["start"] = start
        if strand is not UNSET:
            field_dict["strand"] = strand
        if type is not UNSET:
            field_dict["type"] = type
        if color is not UNSET:
            field_dict["color"] = color
        if custom_fields is not UNSET:
            field_dict["customFields"] = custom_fields
        if name is not UNSET:
            field_dict["name"] = name
        if notes is not UNSET:
            field_dict["notes"] = notes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_end() -> Union[Unset, int]:
            end = d.pop("end")
            return end

        try:
            end = get_end()
        except KeyError:
            if strict:
                raise
            end = cast(Union[Unset, int], UNSET)

        def get_start() -> Union[Unset, int]:
            start = d.pop("start")
            return start

        try:
            start = get_start()
        except KeyError:
            if strict:
                raise
            start = cast(Union[Unset, int], UNSET)

        def get_strand() -> Union[Unset, int]:
            strand = d.pop("strand")
            return strand

        try:
            strand = get_strand()
        except KeyError:
            if strict:
                raise
            strand = cast(Union[Unset, int], UNSET)

        def get_type() -> Union[Unset, str]:
            type = d.pop("type")
            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, str], UNSET)

        def get_color() -> Union[Unset, str]:
            color = d.pop("color")
            return color

        try:
            color = get_color()
        except KeyError:
            if strict:
                raise
            color = cast(Union[Unset, str], UNSET)

        def get_custom_fields() -> Union[Unset, List[SequenceFeatureCustomField]]:
            custom_fields = []
            _custom_fields = d.pop("customFields")
            for custom_fields_item_data in _custom_fields or []:
                custom_fields_item = SequenceFeatureCustomField.from_dict(
                    custom_fields_item_data, strict=False
                )

                custom_fields.append(custom_fields_item)

            return custom_fields

        try:
            custom_fields = get_custom_fields()
        except KeyError:
            if strict:
                raise
            custom_fields = cast(Union[Unset, List[SequenceFeatureCustomField]], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_notes() -> Union[Unset, str]:
            notes = d.pop("notes")
            return notes

        try:
            notes = get_notes()
        except KeyError:
            if strict:
                raise
            notes = cast(Union[Unset, str], UNSET)

        rna_annotation = cls(
            end=end,
            start=start,
            strand=strand,
            type=type,
            color=color,
            custom_fields=custom_fields,
            name=name,
            notes=notes,
        )

        rna_annotation.additional_properties = d
        return rna_annotation

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def end(self) -> int:
        if isinstance(self._end, Unset):
            raise NotPresentError(self, "end")
        return self._end

    @end.setter
    def end(self, value: int) -> None:
        self._end = value

    @end.deleter
    def end(self) -> None:
        self._end = UNSET

    @property
    def start(self) -> int:
        if isinstance(self._start, Unset):
            raise NotPresentError(self, "start")
        return self._start

    @start.setter
    def start(self, value: int) -> None:
        self._start = value

    @start.deleter
    def start(self) -> None:
        self._start = UNSET

    @property
    def strand(self) -> int:
        if isinstance(self._strand, Unset):
            raise NotPresentError(self, "strand")
        return self._strand

    @strand.setter
    def strand(self, value: int) -> None:
        self._strand = value

    @strand.deleter
    def strand(self) -> None:
        self._strand = UNSET

    @property
    def type(self) -> str:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: str) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET

    @property
    def color(self) -> str:
        """ Hex color code used when displaying this feature in the UI. """
        if isinstance(self._color, Unset):
            raise NotPresentError(self, "color")
        return self._color

    @color.setter
    def color(self, value: str) -> None:
        self._color = value

    @color.deleter
    def color(self) -> None:
        self._color = UNSET

    @property
    def custom_fields(self) -> List[SequenceFeatureCustomField]:
        if isinstance(self._custom_fields, Unset):
            raise NotPresentError(self, "custom_fields")
        return self._custom_fields

    @custom_fields.setter
    def custom_fields(self, value: List[SequenceFeatureCustomField]) -> None:
        self._custom_fields = value

    @custom_fields.deleter
    def custom_fields(self) -> None:
        self._custom_fields = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def notes(self) -> str:
        if isinstance(self._notes, Unset):
            raise NotPresentError(self, "notes")
        return self._notes

    @notes.setter
    def notes(self, value: str) -> None:
        self._notes = value

    @notes.deleter
    def notes(self) -> None:
        self._notes = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntryAttachments")


@attr.s(auto_attribs=True, repr=False)
class EntryAttachments:
    """  """

    _attachment_text: Union[Unset, None] = UNSET
    _attachments: Union[Unset, List[str]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("attachment_text={}".format(repr(self._attachment_text)))
        fields.append("attachments={}".format(repr(self._attachments)))
        return "EntryAttachments({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        attachment_text = None

        attachments: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._attachments, Unset):
            attachments = self._attachments

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if attachment_text is not UNSET:
            field_dict["attachmentText"] = attachment_text
        if attachments is not UNSET:
            field_dict["attachments"] = attachments

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_attachment_text() -> Union[Unset, None]:
            attachment_text = None

            return attachment_text

        try:
            attachment_text = get_attachment_text()
        except KeyError:
            if strict:
                raise
            attachment_text = cast(Union[Unset, None], UNSET)

        def get_attachments() -> Union[Unset, List[str]]:
            attachments = cast(List[str], d.pop("attachments"))

            return attachments

        try:
            attachments = get_attachments()
        except KeyError:
            if strict:
                raise
            attachments = cast(Union[Unset, List[str]], UNSET)

        entry_attachments = cls(
            attachment_text=attachment_text,
            attachments=attachments,
        )

        return entry_attachments

    @property
    def attachment_text(self) -> None:
        """Text to include in the entry before the attachments. If not provided, the text will be "File attachments are listed below:" """
        if isinstance(self._attachment_text, Unset):
            raise NotPresentError(self, "attachment_text")
        return self._attachment_text

    @attachment_text.setter
    def attachment_text(self, value: None) -> None:
        self._attachment_text = value

    @attachment_text.deleter
    def attachment_text(self) -> None:
        self._attachment_text = UNSET

    @property
    def attachments(self) -> List[str]:
        """An array of IDs representing Blob objects in Benchling to be attached to the entry."""
        if isinstance(self._attachments, Unset):
            raise NotPresentError(self, "attachments")
        return self._attachments

    @attachments.setter
    def attachments(self, value: List[str]) -> None:
        self._attachments = value

    @attachments.deleter
    def attachments(self) -> None:
        self._attachments = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.request_response_samples_item_batch import RequestResponseSamplesItemBatch
from ..models.request_response_samples_item_entity import RequestResponseSamplesItemEntity
from ..models.request_response_samples_item_status import RequestResponseSamplesItemStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="RequestResponseSamplesItem")


@attr.s(auto_attribs=True, repr=False)
class RequestResponseSamplesItem:
    """  """

    _batch: Union[Unset, None, RequestResponseSamplesItemBatch] = UNSET
    _entity: Union[Unset, None, RequestResponseSamplesItemEntity] = UNSET
    _status: Union[Unset, RequestResponseSamplesItemStatus] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("batch={}".format(repr(self._batch)))
        fields.append("entity={}".format(repr(self._entity)))
        fields.append("status={}".format(repr(self._status)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "RequestResponseSamplesItem({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        batch: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._batch, Unset):
            batch = self._batch.to_dict() if self._batch else None

        entity: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._entity, Unset):
            entity = self._entity.to_dict() if self._entity else None

        status: Union[Unset, int] = UNSET
        if not isinstance(self._status, Unset):
            status = self._status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if batch is not UNSET:
            field_dict["batch"] = batch
        if entity is not UNSET:
            field_dict["entity"] = entity
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_batch() -> Union[Unset, None, RequestResponseSamplesItemBatch]:
            batch = None
            _batch = d.pop("batch")

            if _batch is not None and not isinstance(_batch, Unset):
                batch = RequestResponseSamplesItemBatch.from_dict(_batch)

            return batch

        try:
            batch = get_batch()
        except KeyError:
            if strict:
                raise
            batch = cast(Union[Unset, None, RequestResponseSamplesItemBatch], UNSET)

        def get_entity() -> Union[Unset, None, RequestResponseSamplesItemEntity]:
            entity = None
            _entity = d.pop("entity")

            if _entity is not None and not isinstance(_entity, Unset):
                entity = RequestResponseSamplesItemEntity.from_dict(_entity)

            return entity

        try:
            entity = get_entity()
        except KeyError:
            if strict:
                raise
            entity = cast(Union[Unset, None, RequestResponseSamplesItemEntity], UNSET)

        def get_status() -> Union[Unset, RequestResponseSamplesItemStatus]:
            status = UNSET
            _status = d.pop("status")
            if _status is not None and _status is not UNSET:
                try:
                    status = RequestResponseSamplesItemStatus(_status)
                except ValueError:
                    status = RequestResponseSamplesItemStatus.of_unknown(_status)

            return status

        try:
            status = get_status()
        except KeyError:
            if strict:
                raise
            status = cast(Union[Unset, RequestResponseSamplesItemStatus], UNSET)

        request_response_samples_item = cls(
            batch=batch,
            entity=entity,
            status=status,
        )

        request_response_samples_item.additional_properties = d
        return request_response_samples_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def batch(self) -> Optional[RequestResponseSamplesItemBatch]:
        if isinstance(self._batch, Unset):
            raise NotPresentError(self, "batch")
        return self._batch

    @batch.setter
    def batch(self, value: Optional[RequestResponseSamplesItemBatch]) -> None:
        self._batch = value

    @batch.deleter
    def batch(self) -> None:
        self._batch = UNSET

    @property
    def entity(self) -> Optional[RequestResponseSamplesItemEntity]:
        if isinstance(self._entity, Unset):
            raise NotPresentError(self, "entity")
        return self._entity

    @entity.setter
    def entity(self, value: Optional[RequestResponseSamplesItemEntity]) -> None:
        self._entity = value

    @entity.deleter
    def entity(self) -> None:
        self._entity = UNSET

    @property
    def status(self) -> RequestResponseSamplesItemStatus:
        """ The status of the sample, based on the status of the stage run that generated it. """
        if isinstance(self._status, Unset):
            raise NotPresentError(self, "status")
        return self._status

    @status.setter
    def status(self, value: RequestResponseSamplesItemStatus) -> None:
        self._status = value

    @status.deleter
    def status(self) -> None:
        self._status = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.locations_archive_reason import LocationsArchiveReason
from ..types import UNSET, Unset

T = TypeVar("T", bound="LocationsArchive")


@attr.s(auto_attribs=True, repr=False)
class LocationsArchive:
    """  """

    _location_ids: List[str]
    _reason: LocationsArchiveReason
    _should_remove_barcodes: Union[Unset, bool] = UNSET

    def __repr__(self):
        fields = []
        fields.append("location_ids={}".format(repr(self._location_ids)))
        fields.append("reason={}".format(repr(self._reason)))
        fields.append("should_remove_barcodes={}".format(repr(self._should_remove_barcodes)))
        return "LocationsArchive({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        location_ids = self._location_ids

        reason = self._reason.value

        should_remove_barcodes = self._should_remove_barcodes

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if location_ids is not UNSET:
            field_dict["locationIds"] = location_ids
        if reason is not UNSET:
            field_dict["reason"] = reason
        if should_remove_barcodes is not UNSET:
            field_dict["shouldRemoveBarcodes"] = should_remove_barcodes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_location_ids() -> List[str]:
            location_ids = cast(List[str], d.pop("locationIds"))

            return location_ids

        try:
            location_ids = get_location_ids()
        except KeyError:
            if strict:
                raise
            location_ids = cast(List[str], UNSET)

        def get_reason() -> LocationsArchiveReason:
            _reason = d.pop("reason")
            try:
                reason = LocationsArchiveReason(_reason)
            except ValueError:
                reason = LocationsArchiveReason.of_unknown(_reason)

            return reason

        try:
            reason = get_reason()
        except KeyError:
            if strict:
                raise
            reason = cast(LocationsArchiveReason, UNSET)

        def get_should_remove_barcodes() -> Union[Unset, bool]:
            should_remove_barcodes = d.pop("shouldRemoveBarcodes")
            return should_remove_barcodes

        try:
            should_remove_barcodes = get_should_remove_barcodes()
        except KeyError:
            if strict:
                raise
            should_remove_barcodes = cast(Union[Unset, bool], UNSET)

        locations_archive = cls(
            location_ids=location_ids,
            reason=reason,
            should_remove_barcodes=should_remove_barcodes,
        )

        return locations_archive

    @property
    def location_ids(self) -> List[str]:
        """ Array of location IDs """
        if isinstance(self._location_ids, Unset):
            raise NotPresentError(self, "location_ids")
        return self._location_ids

    @location_ids.setter
    def location_ids(self, value: List[str]) -> None:
        self._location_ids = value

    @property
    def reason(self) -> LocationsArchiveReason:
        """Reason that locations are being archived."""
        if isinstance(self._reason, Unset):
            raise NotPresentError(self, "reason")
        return self._reason

    @reason.setter
    def reason(self, value: LocationsArchiveReason) -> None:
        self._reason = value

    @property
    def should_remove_barcodes(self) -> bool:
        """Remove barcodes. Removing barcodes from archived inventory that contain items will also remove barcodes from the contained items."""
        if isinstance(self._should_remove_barcodes, Unset):
            raise NotPresentError(self, "should_remove_barcodes")
        return self._should_remove_barcodes

    @should_remove_barcodes.setter
    def should_remove_barcodes(self, value: bool) -> None:
        self._should_remove_barcodes = value

    @should_remove_barcodes.deleter
    def should_remove_barcodes(self) -> None:
        self._should_remove_barcodes = UNSET



import datetime
from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="LegacyWorkflow")


@attr.s(auto_attribs=True, repr=False)
class LegacyWorkflow:
    """  """

    _created_at: Union[Unset, datetime.datetime] = UNSET
    _description: Union[Unset, None, str] = UNSET
    _display_id: Union[Unset, str] = UNSET
    _id: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET
    _project_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("created_at={}".format(repr(self._created_at)))
        fields.append("description={}".format(repr(self._description)))
        fields.append("display_id={}".format(repr(self._display_id)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("project_id={}".format(repr(self._project_id)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "LegacyWorkflow({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self._created_at, Unset):
            created_at = self._created_at.isoformat()

        description = self._description
        display_id = self._display_id
        id = self._id
        name = self._name
        project_id = self._project_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if description is not UNSET:
            field_dict["description"] = description
        if display_id is not UNSET:
            field_dict["displayId"] = display_id
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if project_id is not UNSET:
            field_dict["projectId"] = project_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_created_at() -> Union[Unset, datetime.datetime]:
            created_at: Union[Unset, datetime.datetime] = UNSET
            _created_at = d.pop("createdAt")
            if _created_at is not None and not isinstance(_created_at, Unset):
                created_at = isoparse(cast(str, _created_at))

            return created_at

        try:
            created_at = get_created_at()
        except KeyError:
            if strict:
                raise
            created_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_description() -> Union[Unset, None, str]:
            description = d.pop("description")
            return description

        try:
            description = get_description()
        except KeyError:
            if strict:
                raise
            description = cast(Union[Unset, None, str], UNSET)

        def get_display_id() -> Union[Unset, str]:
            display_id = d.pop("displayId")
            return display_id

        try:
            display_id = get_display_id()
        except KeyError:
            if strict:
                raise
            display_id = cast(Union[Unset, str], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_project_id() -> Union[Unset, str]:
            project_id = d.pop("projectId")
            return project_id

        try:
            project_id = get_project_id()
        except KeyError:
            if strict:
                raise
            project_id = cast(Union[Unset, str], UNSET)

        legacy_workflow = cls(
            created_at=created_at,
            description=description,
            display_id=display_id,
            id=id,
            name=name,
            project_id=project_id,
        )

        legacy_workflow.additional_properties = d
        return legacy_workflow

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def created_at(self) -> datetime.datetime:
        """ DateTime at which the the legacy workflow was created """
        if isinstance(self._created_at, Unset):
            raise NotPresentError(self, "created_at")
        return self._created_at

    @created_at.setter
    def created_at(self, value: datetime.datetime) -> None:
        self._created_at = value

    @created_at.deleter
    def created_at(self) -> None:
        self._created_at = UNSET

    @property
    def description(self) -> Optional[str]:
        """ Description of the legacy workflow """
        if isinstance(self._description, Unset):
            raise NotPresentError(self, "description")
        return self._description

    @description.setter
    def description(self, value: Optional[str]) -> None:
        self._description = value

    @description.deleter
    def description(self) -> None:
        self._description = UNSET

    @property
    def display_id(self) -> str:
        """ User-friendly ID of the legacy workflow """
        if isinstance(self._display_id, Unset):
            raise NotPresentError(self, "display_id")
        return self._display_id

    @display_id.setter
    def display_id(self, value: str) -> None:
        self._display_id = value

    @display_id.deleter
    def display_id(self) -> None:
        self._display_id = UNSET

    @property
    def id(self) -> str:
        """ ID of the legacy workflow """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def name(self) -> str:
        """ Name of the legacy workflow """
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def project_id(self) -> str:
        """ ID of the project that contains the legacy workflow """
        if isinstance(self._project_id, Unset):
            raise NotPresentError(self, "project_id")
        return self._project_id

    @project_id.setter
    def project_id(self, value: str) -> None:
        self._project_id = value

    @project_id.deleter
    def project_id(self) -> None:
        self._project_id = UNSET
